@@ -13,9 +13,7 @@ fn filters_minsum(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| {
-        png.raw.filter_image(RowFilter::MinSum, false);
-    });
+    b.iter(|| png.raw.filter_image(RowFilter::MinSum, false));
 }
 
 #[bench]
@@ -23,9 +21,7 @@ fn filters_entropy(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| {
-        png.raw.filter_image(RowFilter::Entropy, false);
-    });
+    b.iter(|| png.raw.filter_image(RowFilter::Entropy, false));
 }
 
 #[bench]
@@ -33,9 +29,7 @@ fn filters_bigrams(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| {
-        png.raw.filter_image(RowFilter::Bigrams, false);
-    });
+    b.iter(|| png.raw.filter_image(RowFilter::Bigrams, false));
 }
 
 #[bench]
@@ -43,9 +37,7 @@ fn filters_bigent(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| {
-        png.raw.filter_image(RowFilter::BigEnt, false);
-    });
+    b.iter(|| png.raw.filter_image(RowFilter::BigEnt, false));
 }
 
 #[bench]
@@ -53,7 +45,5 @@ fn filters_brute(b: &mut Bencher) {
     let input = test::black_box(PathBuf::from("tests/files/rgb_8_should_be_rgb_8.png"));
     let png = PngData::new(&input, &Options::default()).unwrap();
 
-    b.iter(|| {
-        png.raw.filter_image(RowFilter::Brute, false);
-    });
+    b.iter(|| png.raw.filter_image(RowFilter::Brute, false));
 }
