@@ -6,6 +6,7 @@ pub struct AtomicMin {
 }
 
 impl AtomicMin {
+    #[must_use]
     pub fn new(init: Option<usize>) -> Self {
         Self {
             val: AtomicUsize::new(init.unwrap_or(usize::MAX)),
@@ -21,11 +22,6 @@ impl AtomicMin {
         }
     }
 
-    /// Unset value is usize_max
-    pub fn as_atomic_usize(&self) -> &AtomicUsize {
-        &self.val
-    }
-
     /// Try a new value, returning true if it is the new minimum
     pub fn set_min(&self, new_val: usize) -> bool {
         new_val < self.val.fetch_min(new_val, SeqCst)
