@@ -1,12 +1,10 @@
 use libdeflater::*;
 
-use crate::{atomicmin::AtomicMin, PngError, PngResult};
+use crate::{PngError, PngResult};
 
-pub fn deflate(data: &[u8], level: u8, max_size: &AtomicMin) -> PngResult<Vec<u8>> {
+pub fn deflate(data: &[u8], level: u8, max_size: Option<usize>) -> PngResult<Vec<u8>> {
     let mut compressor = Compressor::new(CompressionLvl::new(level.into()).unwrap());
-    let capacity = max_size
-        .get()
-        .unwrap_or_else(|| compressor.zlib_compress_bound(data.len()));
+    let capacity = max_size.unwrap_or_else(|| compressor.zlib_compress_bound(data.len()));
     let mut dest = vec![0; capacity];
     let len = compressor
         .zlib_compress(data, &mut dest)
@@ -30,6 +28,7 @@ pub fn inflate(data: &[u8], out_size: usize) -> PngResult<Vec<u8>> {
     Ok(dest)
 }
 
+#[must_use]
 pub fn crc32(data: &[u8]) -> u32 {
     let mut crc = Crc::new();
     crc.update(data);
