@@ -9,11 +9,13 @@ pub enum PngError {
     TimedOut,
     NotPNG,
     APNGNotSupported,
+    APNGOutOfOrder,
     InvalidData,
     TruncatedData,
     ChunkMissing(&'static str),
     InvalidDepthForType(BitDepth, ColorType),
     IncorrectDataLength(usize, usize),
+    C2PAMetadataPreventsChanges,
     Other(Box<str>),
 }
 
@@ -32,14 +34,17 @@ impl fmt::Display for PngError {
                 f.write_str("Missing data in the file; the file is truncated")
             }
             PngError::APNGNotSupported => f.write_str("APNG files are not (yet) supported"),
-            PngError::ChunkMissing(s) => write!(f, "Chunk {} missing or empty", s),
+            PngError::APNGOutOfOrder => f.write_str("APNG chunks are out of order"),
+            PngError::ChunkMissing(s) => write!(f, "Chunk {s} missing or empty"),
             PngError::InvalidDepthForType(d, ref c) => {
-                write!(f, "Invalid bit depth {} for color type {}", d, c)
+                write!(f, "Invalid bit depth {d} for color type {c}")
             }
             PngError::IncorrectDataLength(l1, l2) => write!(
                 f,
-                "Data length {} does not match the expected length {}",
-                l1, l2
+                "Data length {l1} does not match the expected length {l2}"
+            ),
+            PngError::C2PAMetadataPreventsChanges => f.write_str(
+                "The image contains C2PA manifest that would be invalidated by any file changes",
             ),
             PngError::Other(ref s) => f.write_str(s),
         }
@@ -48,6 +53,7 @@ impl fmt::Display for PngError {
 
 impl PngError {
     #[cold]
+    #[must_use]
     pub fn new(description: &str) -> PngError {
         PngError::Other(description.into())
     }
