@@ -49,7 +49,7 @@ pub fn reduced_to_indexed(png: &PngImage, allow_grayscale: bool) -> Option<PngIm
     let mut raw_data = Vec::with_capacity(png.data.len() / png.channels_per_pixel());
     let palette: Vec<_> = match png.ihdr.color_type {
         ColorType::Grayscale { transparent_shade } => {
-            let pmap = build_palette(png.data.as_gray().iter().cloned(), &mut raw_data)?;
+            let pmap = build_palette(png.data.as_gray().iter().copied(), &mut raw_data)?;
             // Convert the Gray16 transparency to Gray8
             let transparency_pixel = transparent_shade.map(|t| Gray::from(t as u8));
             pmap.into_iter()
@@ -63,7 +63,7 @@ pub fn reduced_to_indexed(png: &PngImage, allow_grayscale: bool) -> Option<PngIm
                 .collect()
         }
         ColorType::RGB { transparent_color } => {
-            let pmap = build_palette(png.data.as_rgb().iter().cloned(), &mut raw_data)?;
+            let pmap = build_palette(png.data.as_rgb().iter().copied(), &mut raw_data)?;
             // Convert the RGB16 transparency to RGB8
             let transparency_pixel = transparent_color.map(|t| t.map(|c| c as u8));
             pmap.into_iter()
@@ -77,11 +77,11 @@ pub fn reduced_to_indexed(png: &PngImage, allow_grayscale: bool) -> Option<PngIm
                 .collect()
         }
         ColorType::GrayscaleAlpha => {
-            let pmap = build_palette(png.data.as_gray_alpha().iter().cloned(), &mut raw_data)?;
+            let pmap = build_palette(png.data.as_gray_alpha().iter().copied(), &mut raw_data)?;
             pmap.into_iter().map(RGBA::from).collect()
         }
         ColorType::RGBA => {
-            let pmap = build_palette(png.data.as_rgba().iter().cloned(), &mut raw_data)?;
+            let pmap = build_palette(png.data.as_rgba().iter().copied(), &mut raw_data)?;
             pmap.into_iter().collect()
         }
         _ => return None,
@@ -106,7 +106,7 @@ pub fn reduced_rgb_to_grayscale(png: &PngImage) -> Option<PngImage> {
     let byte_depth = png.bytes_per_channel();
     let bpp = png.channels_per_pixel() * byte_depth;
     let last_color = 2 * byte_depth;
-    for pixel in png.data.chunks(bpp) {
+    for pixel in png.data.chunks_exact(bpp) {
         if byte_depth == 1 {
             if pixel[0] != pixel[1] || pixel[1] != pixel[2] {
                 return None;
@@ -138,15 +138,30 @@ pub fn reduced_rgb_to_grayscale(png: &PngImage) -> Option<PngImage> {
 
 /// Attempt to convert indexed to a different color type, returning the resulting image if successful
 #[must_use]
-pub fn indexed_to_channels(png: &PngImage, allow_grayscale: bool) -> Option<PngImage> {
+pub fn indexed_to_channels(
+    png: &PngImage,
+    allow_grayscale: bool,
+    optimize_alpha: bool,
+) -> Option<PngImage> {
     if png.ihdr.bit_depth != BitDepth::Eight {
         return None;
     }
-    let palette = match &png.ihdr.color_type {
-        ColorType::Indexed { palette } => palette,
+    let mut palette = match &png.ihdr.color_type {
+        ColorType::Indexed { palette } => palette.clone(),
         _ => return None,
     };
 
+    // Ensure fully transparent colors are black, which can help with grayscale conversion
+    if optimize_alpha {
+        for color in &mut palette {
+            if color.a == 0 {
+                color.r = 0;
+                color.g = 0;
+                color.b = 0;
+            }
+        }
+    }
+
     // Determine which channels are required
     let is_gray = if allow_grayscale {
         palette.iter().all(|c| c.r == c.g && c.g == c.b)
